import BaseController from 'common/base.controller'
import { role, router } from 'decorators/router'
import { ChainTask, ChainTaskClass, TaskStatus } from 'models/ChainTask'
import { ReqTaskStatus, RequestTask } from 'models/RequestTask'
import { ChainQueue } from 'queue/chain.queue'
import { DocumentType } from '@typegoose/typegoose'
import { ConfirmQueue } from 'queue/confirm.queue'
import { BlockChain } from 'chain/BlockChain'

class ChainController extends BaseController {
  @role('anon')
  @router('post /client/login')
  async clientLogin(req, res) {
    const { code, token } = req.params
    return {}
  }

  @role('anon')
  @router('post /chain/req')
  async addChainRequest(req, res) {
    // data是一个数组!!
    const { data, type, taskId, force, max } = req.params
    let maxTryCount = max !== undefined ? parseInt(max) : parseInt(process.env.CHAIN_MAX_TRY)
    let chainTask: DocumentType<ChainTaskClass> = await ChainTask.insertOrUpdate({ taskId }, { type, data })

    if ((!chainTask.newRecord && force) || chainTask.newRecord) {
      for (let sub of data) {
        let subType = sub.type || type
        let subTask = new RequestTask({
          taskId,
          chainTaskId: chainTask.id,
          taskType: subType,
          reqData: sub,
          maxTryCount,
        })
        await subTask.save()
        chainTask.tasks.pushOnce(subTask.id)
        new ChainQueue().addTaskToQueue(subTask)
      }
    }
    await chainTask.save()
    return chainTask.toJson()
  }
}
export default ChainController
